/*
Module : NTRAY.H
Purpose: Interface for a MFC class to encapsulate Shell_NotifyIcon
Created: PJN / 14-05-1997
History: None

Copyright (c) 1997 - 1999 by PJ Naughter.  
All rights reserved.

	SAW 11-11-98	1: ported to ATL by Shaun Wilde shaun.wilde@many-monkeys.freeserve.co.uk
					2: added a magic macro to simulate OnUpdate*** type functionality

	SAW 6-5-99:		: added selectable menu support.

*/


////////////////////////////////// Macros ///////////////////////////

#ifndef _NTRAY_H__
#define _NTRAY_H__

// menu handling class ntray
#define MENU_UPDATE_COMMAND_HANDLER(id, func) \
	if(uMsg == WM_COMMAND && id == LOWORD(wParam) && 0xffff == HIWORD(wParam)) \
	{ \
		bHandled = TRUE; \
		lResult = func(HIWORD(wParam), LOWORD(wParam), (HMENU)lParam, bHandled); \
		if(bHandled) \
			return TRUE; \
	}

/////////////////////////// Classes /////////////////////////////////

//forward declaration
class CTrayNotifyIcon;


//internal class used to handle IE4 taskbar creation notification message
class CTrayIconHooker : public CWindowImpl<CTrayIconHooker>
{
public: 
	DECLARE_WND_CLASS(_T("TrayIconHooker")) 
		
	//Constructors / Destructors
	CTrayIconHooker();
	~CTrayIconHooker();

	void Init(CTrayNotifyIcon* pTrayIcon, CWindow* pNotifyWnd);
	void StartUsingAnimation(HICON* phIcons, int nNumIcons, DWORD dwDelay);
	void StopUsingAnimation();
	BOOL UsingAnimatedIcon() const;
	HICON GetCurrentIcon() const;
			
protected:
	CTrayNotifyIcon* m_pTrayIcon;
	HICON*           m_phIcons;
	int              m_nNumIcons;
	DWORD            m_dwDelay;
	UINT             m_nTimerID;
	int              m_nCurrentIconIndex;

	static UINT wm_TaskbarCreated;

	BEGIN_MSG_MAP(CTrayIconHooker)
		MESSAGE_HANDLER(wm_TaskbarCreated, OnTaskbarCreated)	
		//MESSAGE_HANDLER(WM_CREATE, OnCreate)	
		MESSAGE_HANDLER(WM_DESTROY, OnDestroy)	
		MESSAGE_HANDLER(WM_TIMER, OnTimer)	
	END_MSG_MAP()

	LRESULT OnTaskbarCreated(UINT uMsg,WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	//LRESULT OnCreate(UINT uMsg,WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnDestroy(UINT uMsg,WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnTimer(UINT uMsg,WPARAM wParam, LPARAM lParam, BOOL& bHandled);
};

//the actual tray notification class wrapper
class CTrayNotifyIcon //: public CObject
{
public:
	//Constructors / Destructors
	CTrayNotifyIcon();
	~CTrayNotifyIcon();
	
	//Create the tray icon
	BOOL Create(CWindow* pNotifyWnd, UINT uID, LPCTSTR pszTooltipText, HICON hIcon, UINT nNotifyMessage);                  
	BOOL Create(CWindow* pNotifyWnd, UINT uID, LPCTSTR pszTooltipText, HICON* phIcons, int nNumIcons, DWORD dwDelay, UINT nNotifyMessage);
	
	//Change or retrieve the Tooltip text
	BOOL    SetTooltipText(LPCTSTR pszTooltipText);
	BOOL    SetTooltipText(UINT nID);
	LPCTSTR	GetTooltipText() const;
	
	//Change or retrieve the icon displayed
	BOOL  SetIcon(HICON hIcon);
	BOOL  SetIcon(LPCTSTR lpIconName);
	BOOL  SetIcon(UINT nIDResource);
	BOOL  SetIcon(HICON* phIcons, int nNumIcons, DWORD dwDelay);
	BOOL  SetStandardIcon(LPCTSTR lpIconName);
	BOOL  SetStandardIcon(UINT nIDResource);
	HICON GetIcon() const;
	BOOL  UsingAnimatedIcon() const;
	
	//Change or retrieve the window to send notification messages to
	BOOL  SetNotificationWnd(CWindow* pNotifyWnd);
	CWindow* GetNotificationWnd() const;
	
	//Modification of the tray icons
	void  HideIcon();
	void  ShowIcon();
	void  RemoveIcon();
	void  MoveToExtremeRight();
	
	//Default handler for tray notification message
	virtual LRESULT OnTrayNotification(WPARAM uID, LPARAM lEvent);
	
	//Status information
	BOOL IsShowing() const { return !IsHidden(); };
	BOOL IsHidden() const { return m_bHidden; };

	//Helper functions to load tray icon from resources
	static HICON LoadIconResource(LPCTSTR lpIconName);
	static HICON LoadIconResource(UINT nIDResource);

	// set default menu item
	void SetDefaultMenuItem(UINT uiMenuItem=0);
	UINT GetDefaultMenuItem();

	//Modification of the menu to use as the context menu
	void SetMenu(HMENU hMenu);
	HMENU GetMenu();

protected:

	void ModifyMenu(HMENU hMenu); // performs on_update_ui equiv commands
	
	NOTIFYICONDATA        m_NotifyIconData;
	BOOL                  m_bCreated;
	BOOL				  m_bHidden;
	CWindow*			  m_pNotificationWnd;
	CTrayIconHooker       m_HookWnd;
	HMENU                 m_Menu;

	UINT m_uiDefaultMenuItem;
		
 friend class CTrayIconHooker;
};




#endif //_NTRAY_H__
